@@ -7,12 +7,23 @@ use Time::Piece;
 use Net::Whois::Raw qw(whois $OMIT_MSG $CHECK_FAIL);
 use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION $USE_REGISTRAR_SERVERS);
 
-@ISA = qw(Exporter);
-@EXPORT = qw( expire_date expdate_fmt expdate_int );
-@EXPORT_OK = qw( decode_date );
-$VERSION = '0.24';
+use constant FLG_EXPDATE => 0b0001;
+use constant FLG_CREDATE => 0b0010;
+use constant FLG_ALL     => 0b1111;
 
-$USE_REGISTRAR_SERVERS = 0; # Don't make direct queries to registrar server
+@ISA = qw(Exporter);
+@EXPORT = qw(
+    expire_date expdate_int expdate_fmt credate_fmt domain_dates domdates_fmt
+    $USE_REGISTRAR_SERVERS
+);
+@EXPORT_OK = qw( decode_date );
+$VERSION = '0.33';
+
+$USE_REGISTRAR_SERVERS = 0;
+# 0 - make queries to registry server
+# 1 - make queries to registrar server
+# 2 - make queries to registrar server
+#     and in case of fault make query to registry server
 
 # for Net::Whois::Raw
 $OMIT_MSG = 2; $CHECK_FAIL = 3;
@@ -20,15 +31,55 @@ $OMIT_MSG = 2; $CHECK_FAIL = 3;
 sub expire_date {
     my ($domain, $format) = @_;
 
+    if ($USE_REGISTRAR_SERVERS == 0) {
+	return expire_date_query( $domain, $format, 1 )
+    } elsif ($USE_REGISTRAR_SERVERS == 1) {
+	return expire_date_query( $domain, $format, 0 )
+    } elsif ($USE_REGISTRAR_SERVERS == 2) {
+	return expire_date_query( $domain, $format, 0 )
+	    || expire_date_query( $domain, $format, 1 );
+    }
+    
+    return undef;
+}
+
+sub domain_dates {
+    my ($domain, $format) = @_;
+
     return undef unless ($domain =~ /(.+?)\.([^.]+)$/);
     my ($name, $tld) = (lc $1, lc $2);
 
     my $whois;
-    if (($tld eq 'com' || $tld eq 'net') && !$USE_REGISTRAR_SERVERS) {
+    if (isin($tld, ['com', 'net'])) {
 	$whois = whois( $domain, 'whois.crsnic.net' );
-    } elsif ($tld eq 'org' && !$USE_REGISTRAR_SERVERS) {
+    } elsif ($tld eq 'org') {
+	$whois = whois( $domain, 'whois.publicinterestregistry.net' );
+    } else {
+	$whois = whois( $domain );
+    }
+
+    if ($format) {
+	return (domdates_fmt( $whois, $tld, $format ));
+    } else {
+	return (domdates_int( $whois, $tld ));
+    }
+    
+    return undef;
+}
+
+sub expire_date_query {
+    my ($domain, $format, $via_registry) = @_;
+
+    return undef unless ($domain =~ /(.+?)\.([^.]+)$/);
+    my ($name, $tld) = (lc $1, lc $2);
+
+    my $whois;
+    if (isin($tld, ['com', 'net']) && $via_registry) {
+	$whois = whois( $domain, 'whois.crsnic.net' );
+    } elsif ($tld eq 'org' && $via_registry) {
 	$whois = whois( $domain, 'whois.publicinterestregistry.net' );
     }
+
     $whois ||= whois( $domain );
 
     if ($format) {
@@ -38,30 +89,56 @@ sub expire_date {
     }
 }
 
-sub expdate_fmt {
-    my ($whois, $tld, $format) = @_;
+sub domdates_fmt {
+    my ($whois, $tld, $format, $flags) = @_;
     $format ||= '%Y-%m-%d';
 
-    my $time = expdate_int( $whois, $tld );
-    return undef unless $time;
-    
-    local $^W=0;                # prevent a warning
-    return $time->strftime( $format );
+    my ($cre_date, $exp_date, $fre_date) = domdates_int( $whois, $tld, $flags );
+
+    local $^W = 0;  # prevent warnings
+
+    $cre_date = $cre_date ? $cre_date->strftime( $format ) : '';
+    $exp_date = $exp_date ? $exp_date->strftime( $format ) : '';
+    $fre_date = $fre_date ? $fre_date->strftime( $format ) : '';
+
+    return ($cre_date, $exp_date, $fre_date);
+}
+
+sub expdate_fmt {
+    my ($whois, $tld, $format) = @_;
+
+    my ($cre_date, $exp_date) = domdates_fmt( $whois, $tld, $format, FLG_EXPDATE );
+
+    return $exp_date;
+}
+
+sub credate_fmt {
+    my ($whois, $tld, $format) = @_;
+
+    my ($cre_date, $exp_date) = domdates_fmt( $whois, $tld, $format, FLG_CREDATE );
+
+    return $cre_date;
+}
+
+sub domdates_int {
+    my ($whois, $tld, $flags) = @_;
+    $tld ||= 'com';
+    $flags ||= FLG_ALL;
+
+    if ($tld eq 'ru' || $tld eq 'su') {
+	return (dates_int_ru( $whois ));
+    } else { # 'com', 'net', 'org', 'biz', 'info', 'us', 'uk', 'cc'
+	my $expdate = $flags & FLG_EXPDATE ? expdate_int_cno( $whois ) : undef;
+	my $credate = $flags & FLG_CREDATE ? credate_int_cno( $whois ) : undef;
+	return ($credate, $expdate);
+    }
 }
 
 sub expdate_int {
     my ($whois, $tld) = @_;
-    $tld ||= 'com';
 
-    if ($tld eq 'ru' || $tld eq 'su') {
-	return expdate_int_ru( $whois );
-    } elsif ($tld eq 'ua') {
-	return expdate_int_ua( $whois );
-    } elsif (isin($tld, ['com', 'net', 'org', 'biz', 'info', 'us'])) {
-	return expdate_int_cno( $whois );
-    } else {
-	return undef;
-    }
+    my ($cre_date, $exp_date, $fre_date) = domdates_int( $whois, $tld, 1 );
+    return $exp_date;
 }
 
 # --- internal functions ----
@@ -95,7 +172,6 @@ sub expdate_int_cno {
 
     # [whois.networksolutions.com]	Record expires on 27-Apr-2011.
     # [whois.opensrs.net]
-    # [whois.easyspace.com]
     # [whois.namesdirect.com]
     # [whois.dotregistrar.com]
     # [whois.domaininfo.com]		Domain expires: 24 Oct 2010
@@ -106,7 +182,8 @@ sub expdate_int_cno {
     # [whois.discount-domain.com]	Expiration Date: 02-Aug-2003 22:07:21
     # [whois.publicinterestregistry.net] Expiration Date:03-Mar-2004 05:00:00 UTC
     # [whois.crsnic.net]		Expiration Date: 21-sep-2004
-    } elsif ($whois =~ m/Expiration Date:\s*(\d{2})-(\w{3})-(\d{4})/s) {
+    # [whois.nic.uk]			Renewal Date:   23-Jan-2006
+    } elsif ($whois =~ m/(?:Expiration|Renewal) Date:\s*(\d{2})-(\w{3})-(\d{4})/s) {
 	$rulenum = 1.2;	$d = $1; $b = $2; $Y = $3;
     # [whois.bulkregister.com]		Record expires on 2003-04-25
     # [whois.bulkregister.com]		Record will be expiring on date: 2003-04-25
@@ -123,7 +200,8 @@ sub expdate_int_cno {
     # [whois.totalnic.net]		Record expires on 2010-04-24 16:03:20+10
     # [whois.signaturedomains.com]	Expires on: 2003-11-05
     # [whois.1stdomain.net]		Domain expires: 2007-01-20.
-    } elsif ($whois =~ m&(?:Record |Domain )?(?:will )?(?:be )?expir(?:e|ed|es|ing)(?: on)?(?: date)?\s*[-:]?\s+(\d{4})[/-](\d{1,2})[/-](\d{2})&is) {
+    # [whois.easyspace.com]
+    } elsif ($whois =~ m&(?:Record |Domain )?(?:will )?(?:be )?expir(?:e|ed|es|ing)(?: on)?(?: date)?\s*[-:]?\s*(\d{4})[/-](\d{1,2})[/-](\d{1,2})&is) {
 	$rulenum = 2.1;	$Y = $1; $m = $2; $d = $3;
     # [whois.InternetNamesWW.com]	Expiry Date.......... 2009-06-16
     # [whois.aitdomains.com]		Expire on................ 2002-11-05 16:42:41.000
@@ -134,7 +212,8 @@ sub expdate_int_cno {
     } elsif ($whois =~ m&(?:Expiry Date|Expire(?:d|s)? on|Valid Date|Expiration Date|Date of expiration)(?:\.*|\s*):?\s+(\d{4})-(\d{2})-(\d{2})&s) {
 	$rulenum = 2.2;	$Y = $1; $m = $2; $d = $3;
     # [whois.oleane.net]		expires:        20030803
-    } elsif ($whois =~ m/expires:\s+(\d{4})(\d{2})(\d{2})/is) {
+    # [whois.nic.it]			expire:      20051011
+    } elsif ($whois =~ m/expires?:\s+(\d{4})(\d{2})(\d{2})/is) {
 	$rulenum = 2.3;	$Y = $1; $m = $2; $d = $3;
     # [whois.dotster.com]		Expires on: 12-DEC-05
     } elsif ($whois =~ m/Expires on: (\d{2})-(\w{3})-(\d{2})/s) {
@@ -158,12 +237,15 @@ sub expdate_int_cno {
     # [whois.psi-domains.com]
     # [whois.namesecure.com]		Expires on 10-09-2011
     # [whois.catalog.com]		Record Expires on 08-24-2011
-    } elsif ($whois =~ m&(?:Record |Domain )?expire(?:d|s) on (\d{2})-(\d{2})-(\d{4})&is) {
+    } elsif ($whois =~ m&expires.+?(\d{2})-(\d{2})-(\d{4})&is) {
 	$rulenum = 5.1;	$m = $1; $d = $2; $Y = $3;
     # [whois.stargateinc.com]		Expiration: 6/3/2004
     # [whois.bookmyname.com]		Expires on 11/26/2007 23:00:00
     } elsif ($whois =~ m&(?:Expiration|Expires on):? (\d{1,2})[-/](\d{1,2})[-/](\d{4})&is) {
 	$rulenum = 5.2;	$m = $1; $d = $2; $Y = $3;
+    # [whois.belizenic.bz]		Expiration Date..: 15-01-2005 12:00:00
+    } elsif ($whois =~ m&Expiration Date.+?(\d{2})-(\d{2})-(\d{4}) \d{2}:\d{2}:\d{2}&is) {
+	$rulenum = 5.3;	$d = $1; $m = $2; $Y = $3;
     # [whois.nordnet.net]		Record expires on 2010-Apr-03
     # [whois.alldomains.com]		Expires on..............: 2006-Jun-12
     } elsif ($whois =~ m/(?:Record |Domain )?expires on\.*:? (\d{4})-(\w{3})-(\d{2})/is) {
@@ -180,7 +262,7 @@ sub expdate_int_cno {
     }
 
     unless ($rulenum) {
-	warn "Can't recognise date format\n";
+	warn "Can't recognise expiration date format\n";
 	return undef;
     } else {
 	#warn "rulenum: $rulenum\n";
@@ -199,60 +281,90 @@ sub expdate_int_cno {
     return decode_date( $dstr, $fstr );
 }
 
-# extract expiration date from whois output for .ru and .su domains
-sub expdate_int_ru {
+
+# extract creation date from whois output for .com .net .org domains
+sub credate_int_cno {
     my ($whois) = @_;
     return undef unless $whois;
 
-    my @states;
-    while ($whois =~ /state:   (.+?)\n/gs) { pushstate(\@states, $1) };
-    while ($whois =~ /reg-till: (.+?)\n/gs) { pushstate(\@states, "reg-till: $1") };
-    while ($whois =~ /payed-till: (.+?)\n/gs) { pushstate(\@states, "reg-till: $1") };
-    while ($whois =~ /paid-till: (.+?)\n/gs) { pushstate(\@states, "reg-till: $1") };
-    while ($whois =~ /free-date:(.+?)\n/gs) { pushstate(\@states, "free-date: $1") };
-    my $res = join( '; ', @states );
+    # $Y - The year, including century
+    # $y - The year within century (0-99)
+    # $m - The month number (1-12)
+    # $b - The month name
+    # $d - The day of month (1-31)
+    my ($rulenum, $Y, $y, $m, $b, $d);
 
-    my ($reg_till, $free_date, $active, $created);
-
-    # ON-HOLD domains
-    if ($res =~ /NOT DELEGATED; reg-till:\s+([0-9.]+); free-date:\s+([0-9.]+)/) {
-	$active = 0;
-	($reg_till = $1) =~ tr/./-/;
-	($free_date = $2) =~ tr/./-/;
-    } elsif ($res =~ /NOT DELEGATED; reg-till: ([0-9.]+)/i) {
-	$active = 0;
-	($reg_till = $1) =~ tr/./-/;
-    } elsif ($res =~ /Not delegated; (?:freeing date -|free-date:)\s+([0-9.]+)/i) {
-	$active = 0;
-	($free_date = $1) =~ tr/./-/;
-    } elsif ($res =~ /Not delegated/i) {
-	$active = 0;
-    # ACTIVE DOMAINS
-    } elsif ($res =~ /reg-till:\s+([0-9.]+); free-date:\s+([0-9.]+)/) {
-	$active = 1;
-	($reg_till = $1) =~ tr/./-/;
-	($free_date = $2) =~ tr/./-/;
-    } elsif ($res =~ /reg-till:\s+([0-9.]+)/) {
-	$active = 1;
-	($reg_till = $1) =~ tr/./-/;
-    } elsif ($res =~ /Delegated till ([0-9.]+)/) {
-	$active = 1;
-	($reg_till = $1) =~ tr/./-/;
-    } elsif ($res =~ /Delegated/) {
-	$active = 1;
-    # NOT-ACTIVE-YET DOMAINS
-    } elsif ($res =~ /RIPN NCC check in progress/) {
-	$active = 0;
+    # [whois.crsnic.net]		Creation Date: 06-sep-2000
+    # [whois.afilias.info]		Created On:31-Jul-2001 08:42:21 UTC
+    if ($whois =~ m/Creat.+?:\s*(\d{2})-(\w{3})-(\d{4})/s) {
+	$rulenum = 1.2;	$d = $1; $b = $2; $Y = $3;
+    # [whois.nic.name]			Created On: 2002-02-08T14:56:54Z
+    # [whois.worldsite.ws]		Domain created on 2002-10-29 03:54:36
+    } elsif ($whois =~ m/Creat.+?:?\s*?(\d{4})[\/-](\d{1,2})[\/-](\d{1,2})/is) {
+	$rulenum = 2.1;	$Y = $1; $m = $2; $d = $3;
+    # [whois.nic.it]			created:     20000421
+    } elsif ($whois =~ m/created?:\s+(\d{4})(\d{2})(\d{2})/is) {
+	$rulenum = 2.3;	$Y = $1; $m = $2; $d = $3;
+    # [whois.relcom.net]		changed:      support@webnames.ru 20030815
+    } elsif ($whois =~ m/changed:.+?(\d{4})(\d{2})(\d{2})/is) {
+	$rulenum = 2.4;	$Y = $1; $m = $2; $d = $3;
+    # [whois.tv]			Record created on Feb 21 2001.
+    } elsif ($whois =~ m/Creat.+?:?\s*(?:\w{3}, )?(\w{3,9})\s{1,2}(\d{1,2}),? (\d{4})/is) {
+	$rulenum = 4.1;	$b = $1; $d = $2; $Y = $3;
+    # [whois.dns.be]			Registered:  Wed Jan 17 2001
+    } elsif ($whois =~ m/Regist.+?:\s*\w{3} (\w{3})\s+(\d{1,2}) (?:\d{2}:\d{2}:\d{2} )?(\d{4})/is) {
+	$rulenum = 4.2;	$b = $1; $d = $2; $Y = $3;
+    # [whois.whois.neulevel.biz]	Domain Registration Date: Wed Mar 27 00:01:00 GMT 2002
+    } elsif ($whois =~ m/Registration.*?:\s+\w{3} (\w{3}) (\d{2}) (?:\d{2}:\d{2}:\d{2} \w{3}(?:[-+]\d{2}:\d{2})? )?(\d{4})/is) {
+	$rulenum = 4.3; $b = $1; $d = $2; $Y = $3;
+    } elsif ($whois =~ m&created.+?(\d{2})-(\d{2})-(\d{4})&is) {
+	$rulenum = 5.1;	$m = $1; $d = $2; $Y = $3;
+    # [whois.belizenic.bz]		Creation Date....: 15-01-2003 05:00:00
+    } elsif ($whois =~ m&Creation Date.+?(\d{2})-(\d{2})-(\d{4}) \d{2}:\d{2}:\d{2}&is) {
+	$rulenum = 5.3;	$d = $1; $m = $2; $Y = $3;
     } else {
-	warn "Unknown record: $res\n";
+	warn "Can't recognise creation date format\n";
 	return undef;
     }
 
-    if ($whois =~ /created:\s+([0-9.]+)\n/gs) {
-	($created = $1) =~ tr/./-/;
+    my ($fstr, $dstr) = ('', '');
+    $fstr .= $Y ? '%Y ' : '%y ';
+    $dstr .= $Y ? "$Y " : "$y ";
+
+    $fstr .= $b ? '%b ' : '%m ';
+    $dstr .= $b ? "$b " : "$m ";
+
+    $fstr .= '%d';
+    $dstr .= $d;
+
+    return decode_date( $dstr, $fstr );
+}
+
+
+
+# extract creation/expiration dates from whois output for .ru and .su domains
+sub dates_int_ru {
+    my ($whois) = @_;
+    return undef unless $whois;
+
+    my ($reg_till, $free_date, $created);
+
+    if ($whois =~ /reg-till:\s*(.+?)\n/s) { $reg_till = $1; }
+    if ($whois =~ /Delegated till\s*(.+?)\n/s) { $reg_till = $1; }
+    if ($whois =~ /payed-till:\s*(.+?)\n/s) { $reg_till = $1; }
+    if ($whois =~ /paid-till:\s*(.+?)\n/s) { $reg_till = $1; }
+    if ($whois =~ /free-date:\s*(.+?)\n/s) { $free_date = $1; }
+    if ($whois =~ /created:\s+([0-9.]+)\n/s) { $created = $1; }
+
+    $reg_till =~ tr/./-/ if $reg_till;
+    $free_date =~ tr/./-/ if $free_date;
+    $created =~ tr/./-/ if $created;
+
+    if ($created) {
+	# Guess reg-till date
+	$created = decode_date( $created, '%Y-%m-%d' );
+	my $t = $created;
 
-	#   reg-till
-	my $t = decode_date( $created, '%Y-%m-%d' );
 	if ($t && !$reg_till && !$free_date) {
 	    $t += 0;
 	    while ($t < localtime()) {
@@ -263,7 +375,7 @@ sub expdate_int_ru {
     }
 
     unless ( $reg_till || $free_date ) {
-	warn "Can't obtain expiration date from: $res\n";
+	warn "Can't obtain expiration date from ($reg_till)\n";
 	return undef;
     }
 
@@ -273,33 +385,7 @@ sub expdate_int_ru {
 	$reg_till = $free_date - 33 * ONE_DAY;
     }
     
-    return $reg_till;
-}
-
-sub pushstate {
-    my ($states, $state) = @_;
-    return if (
-	$state =~ /REGISTERED, DELEGATED/i
-	||
-	$state =~ /RIPN NCC check completed OK/i
-    );
-    push @{$states}, $state;
-}
-
-# extract expiration date from whois output for .ua domains
-sub expdate_int_ua {
-    my ($whois) = @_;
-    return undef unless $whois;
-
-    # $Y - The year, including century
-    # $m - The month number (1-12)
-    # $d - The day of month (1-31)
-    my ($rulenum, $Y, $y, $m, $b, $d);
-    if ($whois =~ m/status:\s+OK-UNTIL (\d{4})(\d{2})(\d{2})/is) {
-	$Y = $1; $m = $2; $d = $3;
-	return decode_date( "$Y $m $d", "%Y %m %d" );
-    }
-    return undef;
+    return ($created, $reg_till, $free_date);
 }
 
 sub isin {
@@ -323,10 +409,14 @@ Net::Domain::ExpireDate - obtain expiration date of domain names
 
  use Net::Domain::ExpireDate;
 
- $date = expire_date( 'microsoft.com' );
- $str  = expire_date( 'microsoft.com', '%Y-%m-%d' );
- $date = expdate_int( $whois_text, 'com' );
- $str  = expdate_fmt( $whois_text, 'ru', '%Y-%m-%d' );
+ $expiration_obj = expire_date( 'microsoft.com' );
+ $expiration_str  = expire_date( 'microsoft.com', '%Y-%m-%d' );
+ $expiration_obj = expdate_int( $whois_text, 'com' );
+ $expiration_str  = expdate_fmt( $whois_text, 'ru', '%Y-%m-%d' );
+
+ ($creation_obj, $expiration_obj) = domain_dates( 'microsoft.com' );
+ ($creation_str, $expiration_str) = domain_dates( 'microsoft.com', '%Y-%m-%d' );
+ ($creation_obj, $expiration_obj) = domdates_int( $whois_text, 'com' );
 
 =head1 DESCRIPTION
 
@@ -336,9 +426,8 @@ Unfortunately there are too many different whois servers which provides
 whois info in very different formats.
 Net::Domain::ExpireDate knows more than 40 different formats of
 expiration date representation provided by different servers (almost
-all gTLD registrars and some ccTLD registrars are covered). If an
-expiration date format is unknown to Net::Domain::ExpireDate - then
-heuristics is used to determine expiration date.
+all gTLD registrars and some ccTLD registrars are covered).
+Now obtaining of domain creation date is also supported.
 
 "$date" in synopsis is an object of type L<Time::Piece>.
 
@@ -357,8 +446,8 @@ See L<strftime> man page for C<FORMAT> specification.
 
 Extracts expiration date of domain in TLD from C<WHOISTEXT>.
 If no TLD is given 'com' is the default. There is no
-distinction between 'com', 'net' or 'org' TLDs in this function.
-Also 'biz', 'info', 'us', 'ru' and 'su' TLDs are suppored.
+distinction between 'com' or 'net' TLDs in this function.
+Also 'org', 'biz', 'info', 'us', 'ru' and 'su' TLDs are suppored.
 Returns L<Time::Piece> object.
 
 With C<FORMAT> argument returns date formatted using C<FORMAT> template
@@ -370,11 +459,27 @@ Similar to expdate_int except that output value is formatted date.
 If no C<FORMAT> specified, '%Y-%m-%d' is assumed.
 See L<strftime> man page for C<FORMAT> specification.
 
+=item domain_dates( DOMAIN [,FORMAT] )
+
+Returns list of two values - creation and expiration date of C<DOMAIN>.
+Without C<FORMAT> argument returns L<Time::Piece> objects.
+With C<FORMAT> argument dates are formatted using C<FORMAT> template.
+See L<strftime> man page for C<FORMAT> specification.
+
+=item domdates_int( WHOISTEXT [,TLD] )
+
+Returns list of two values - creation and expiration date of domain
+extracted from C<WHOISTEXT>.
+If no TLD is given 'com' is the default. There is no
+distinction between 'com' or 'net' TLDs in this function.
+Also 'org', 'biz', 'info', 'us', 'ru' and 'su' TLDs are suppored.
+Returns L<Time::Piece> object.
+
 =back
 
 =head1 AUTHOR
 
-Walery Studennikov, <despair@regtime.net>
+Walery Studennikov, <despair@cpan.org>
 
 =head1 SEE ALSO
 
